import { DeepClient } from '@deep-foundation/deeplinks/imports/client';
import { Link } from '@deep-foundation/deeplinks/imports/minilinks';
import { MotionInfo } from './motion-info';
/**
 * Saves {@link SaveMotionInfoParam.info} to the value of a link of type `Motion` which from and to will be {@link SaveMotionInfoParam.deviceLink} or {@link SaveMotionInfoParam.deviceLinkId} depending on which one is passed
 *
 * @remarks
 * After value of Motion link is updated, a handler from `@freephoenix888/object-to-links-async-converter` will create links to represent the value
 *
 * @throws if neither {@link deviceLink} nor {@link deviceLinkId} is passed
 * @throws if {@link deviceLink} is passed but it does not exist
 * @throws if {@link deviceLinkId} is passed but it does not exist
 * @throws if both {@link deviceLink} and {@link deviceLinkId} are passed
 *
 * @example
#### Subscribe to acceleration changes by using this library and save them
```ts
const accelerationHandler = await subscribeToAccelerationChanges({
deep,
deviceLinkId,
});
```
#### Subscribe to orientation changes by using this library and save them
```ts
const newOrientationHandler = await subscribeToOrientationChanges({
deep,
deviceLinkId,
});
```
#### Subscribe to acceleration changes by using @capacitor/motion library and save them
```ts
const accelerationListener = await Motion.addListener(
'accel',
async (accelData) => {
  await saveMotionInfo({
    deep,
    deviceLinkId,
    info: accelData,
  });
}
);
```
#### Subscribe to orientation changes by using @capacitor/motion library and save them
```ts
const accelerationListener = await Motion.addListener(
'orientation',
async (orientationData) => {
  await saveMotionInfo({
    deep,
    deviceLinkId,
    info: {
      rotationRate: orientationData,
    },
  });
}
);
#### Save motion info manually
```ts
let info = {
acceleration: {
  x: 3.2,
  y: 4.7,
  z: -9.8,
},
accelerationIncludingGravity: {
  x: 0.3,
  y: 0.2,
  z: -9.8,
},
rotationRate: {
  alpha: 30,
  beta: 60,
  gamma: 90,
},
interval: 1000,
};

await saveMotionInfo({
deep,
deviceLinkId,
info,
});
```
 */
export declare function saveMotionInfo(params: SaveMotionInfoParam): Promise<void>;
export type SaveMotionInfoParam = {
    deep: DeepClient;
    info: Partial<MotionInfo>;
} & ({
    deviceLinkId: number;
} | {
    deviceLink: Link<number>;
});
