var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { PACKAGE_NAME } from './package-name';
import { createSerialOperation } from '@deep-foundation/deeplinks/imports/gql';
/**
 * Saves {@link SaveMotionInfoParam.info} to the value of a link of type `Motion` which from and to will be {@link SaveMotionInfoParam.deviceLink} or {@link SaveMotionInfoParam.deviceLinkId} depending on which one is passed
 *
 * @remarks
 * After value of Motion link is updated, a handler from `@freephoenix888/object-to-links-async-converter` will create links to represent the value
 *
 * @throws if neither {@link deviceLink} nor {@link deviceLinkId} is passed
 * @throws if {@link deviceLink} is passed but it does not exist
 * @throws if {@link deviceLinkId} is passed but it does not exist
 * @throws if both {@link deviceLink} and {@link deviceLinkId} are passed
 *
 * @example
#### Subscribe to acceleration changes by using this library and save them
```ts
const accelerationHandler = await subscribeToAccelerationChanges({
deep,
deviceLinkId,
});
```
#### Subscribe to orientation changes by using this library and save them
```ts
const newOrientationHandler = await subscribeToOrientationChanges({
deep,
deviceLinkId,
});
```
#### Subscribe to acceleration changes by using @capacitor/motion library and save them
```ts
const accelerationListener = await Motion.addListener(
'accel',
async (accelData) => {
  await saveMotionInfo({
    deep,
    deviceLinkId,
    info: accelData,
  });
}
);
```
#### Subscribe to orientation changes by using @capacitor/motion library and save them
```ts
const accelerationListener = await Motion.addListener(
'orientation',
async (orientationData) => {
  await saveMotionInfo({
    deep,
    deviceLinkId,
    info: {
      rotationRate: orientationData,
    },
  });
}
);
#### Save motion info manually
```ts
let info = {
acceleration: {
  x: 3.2,
  y: 4.7,
  z: -9.8,
},
accelerationIncludingGravity: {
  x: 0.3,
  y: 0.2,
  z: -9.8,
},
rotationRate: {
  alpha: 30,
  beta: 60,
  gamma: 90,
},
interval: 1000,
};

await saveMotionInfo({
deep,
deviceLinkId,
info,
});
```
 */
export function saveMotionInfo(params) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const info = yield removeRedundantFieldsFromMotionInfo({ info: params.info });
        console.log({ params });
        const { deep } = params;
        const serialOperations = [];
        let deviceLink = yield getDeviceLink();
        let motionLinkId;
        let value;
        const motionLink = yield getMotionLinkOrUndefined({
            deviceLinkId: deviceLink.id,
        });
        if (motionLink) {
            motionLinkId = motionLink.id;
            value = (_a = motionLink.value) === null || _a === void 0 ? void 0 : _a.value;
        }
        else {
            const reservedLinkIds = yield deep.reserve(1);
            motionLinkId = reservedLinkIds.pop();
            serialOperations.push(yield getMotionLinkInsertSerialOperation({
                deviceLinkId: deviceLink.id,
                motionLinkId,
            }));
        }
        if (value) {
            serialOperations.push(yield getMotionLinkValueUpdateSerialOperation({ motionLinkId }));
        }
        else {
            serialOperations.push(yield getMotionLinkValueInsertSerialOperation({ motionLinkId }));
        }
        console.log({ serialOperations });
        yield deep.serial({
            operations: serialOperations,
        });
        /**
         * Gets link of type {@link Device}
         *
         * @throws if neither {@link deviceLink} nor {@link deviceLinkId} is passed
         * @throws if {@link deviceLink} is passed but it does not exist
         * @throws if {@link deviceLinkId} is passed but it does not exist
         */
        function getDeviceLink() {
            return __awaiter(this, void 0, void 0, function* () {
                let deviceLink;
                if ('deviceLinkId' in params && 'deviceLink' in params) {
                    throw new Error(`Both deviceLinkId and deviceLink are passed. Either deviceLink or deviceLinkId must be passed`);
                }
                else if ('deviceLinkId' in params) {
                    const { data } = yield deep.select({
                        id: params.deviceLinkId,
                    });
                    deviceLink = data[0];
                }
                else if ('deviceLink' in params) {
                    deviceLink = params.deviceLink;
                }
                else {
                    throw new Error(`Either deviceLink or deviceLinkId must be passed`);
                }
                return deviceLink;
            });
        }
        /**
         * Gets link of type {@link Motion} with id {@link motionLinkId} or undefined if it does not exist
         */
        function getMotionLinkOrUndefined({ deviceLinkId, }) {
            return __awaiter(this, void 0, void 0, function* () {
                const selectData = {
                    type_id: {
                        _id: [PACKAGE_NAME, 'Motion'],
                    },
                    from_id: deviceLinkId,
                    to_id: deviceLinkId,
                };
                const { data: [motionLink], } = yield deep.select(selectData);
                // if (!motionLink) {
                //   throw new Error(`Select with data ${selectData} return empty result`)
                // }
                return motionLink;
            });
        }
        /**
         * Gets serial operation that inserts link of type {@link Motion} with id {@link motionLinkId} to {@link deviceLinkId}
         */
        function getMotionLinkInsertSerialOperation({ deviceLinkId, motionLinkId, }) {
            return __awaiter(this, void 0, void 0, function* () {
                return createSerialOperation({
                    table: 'links',
                    type: 'insert',
                    objects: {
                        id: motionLinkId,
                        type_id: yield deep.id(PACKAGE_NAME, 'Motion'),
                        from_id: deviceLinkId,
                        to_id: deviceLinkId,
                        in: {
                            data: {
                                type_id: yield deep.id('@deep-foundation/core', 'Contain'),
                                from_id: deviceLinkId,
                                string: {
                                    data: {
                                        value: 'Motion',
                                    },
                                },
                            },
                        },
                    },
                });
            });
        }
        /**
         * Gets serial operation that inserts value of a link of type {@link Motion} with id {@link motionLinkId} to {@link info}
         */
        function getMotionLinkValueInsertSerialOperation({ motionLinkId, }) {
            return __awaiter(this, void 0, void 0, function* () {
                return createSerialOperation({
                    table: 'objects',
                    type: 'insert',
                    objects: {
                        link_id: motionLinkId,
                        value: info,
                    },
                });
            });
        }
        /**
         * Gets serial operation that updates value of a link of type {@link Motion} with id {@link motionLinkId} to {@link info}
         */
        function getMotionLinkValueUpdateSerialOperation({ motionLinkId, }) {
            return __awaiter(this, void 0, void 0, function* () {
                return createSerialOperation({
                    table: 'objects',
                    type: 'update',
                    exp: {
                        link_id: motionLinkId,
                    },
                    value: {
                        value: Object.assign({}, info),
                    },
                });
            });
        }
        /**
         * Removes redundant fields from an object of type {@link MotionInfo} because they contain extra fields that are not specified in their typescript interface
         */
        function removeRedundantFieldsFromMotionInfo({ info }) {
            return __awaiter(this, void 0, void 0, function* () {
                // capacitor-motion actually actually pass to us object with extra fields that are not specified in their typescript interface
                return Object.assign(Object.assign(Object.assign(Object.assign({}, ('acceleration' in info
                    ? {
                        acceleration: {
                            x: info.acceleration.x,
                            y: info.acceleration.y,
                            z: info.acceleration.z,
                        },
                    }
                    : {})), ('accelerationIncludingGravity' in info
                    ? {
                        accelerationIncludingGravity: {
                            x: info.accelerationIncludingGravity.x,
                            y: info.accelerationIncludingGravity.y,
                            z: info.accelerationIncludingGravity.z,
                        },
                    }
                    : {})), ('rotationRate' in info
                    ? {
                        rotationRate: {
                            alpha: info.rotationRate.alpha,
                            beta: info.rotationRate.beta,
                            gamma: info.rotationRate.gamma,
                        },
                    }
                    : {})), ('interval' in info ? { interval: info.interval } : {}));
            });
        }
    });
}
